import hashlib
from hmac import compare_digest

class SecurityUtils():

	def hash(self, strInput):
		h = hashlib.sha512(strInput.encode())
		return h.hexdigest()
		
	def getUserHashPassword(self, userId, password):
		strInput = userId + password	
		return self.hash(strInput)

	def verify(self, userId, password, hexdigest):
		hashHexdigest = self.getUserHashPassword(userId, password)
		return compare_digest(hashHexdigest, hexdigest)
		


from flask import Flask
from datetime import datetime
import uuid

from cerberus.model.KsmConnectionModel import KsmConnectionModel
from cerberus.responses.Response import Response
from cerberus.dtos.Connection import Connection
from cerberus.mappers.ConnectionMapper import ConnectionMapper
from cerberus.exceptions.exceptions import InvalidClientConnectionException, ClientConnectionExpiredException
from cerberus.services.AbstractService import AbstractService
from cerberus.exceptions.exceptions import NullClientConnectionException

class ConnectionService(AbstractService):

    def __init__(self, url):
        super(ConnectionService, self).__init__(url)

    def removeConnection(self, token):
        if token is None:
            raise NullClientConnectionException()

        ksmConnection = KsmConnectionModel(self.urlEngine).get(token)
        if ksmConnection is None:
            raise InvalidClientConnectionException()

        ksmConnection.setUpdatedAt(datetime.now())
        ksmConnection.setActive(False)
        KsmConnectionModel(self.urlEngine).update(ksmConnection)
        return ConnectionMapper.mapToConnection(ksmConnection)

    def createConnection(self, client):
        token = str(uuid.uuid4())
        ksmConnection = KsmConnectionModel(self.urlEngine).add(token, client)
        connection = Connection(ksmConnection.getId(), ksmConnection.getClientId(), ksmConnection.getActive(), ksmConnection.getCreatedAt(), ksmConnection.getUpdatedAt())
        return ConnectionMapper.mapToConnection(ksmConnection)

    def validateConnection(self, token):

        if token is None:
            raise NullClientConnectionException()

        ksmConnection = KsmConnectionModel(self.urlEngine).get(token)
        if ksmConnection is None:
            raise InvalidClientConnectionException()

        if ksmConnection.getActive() == False:
            raise ClientConnectionExpiredException()

        ksmConnection.setUpdatedAt(datetime.now())
        KsmConnectionModel(self.urlEngine).update(ksmConnection)
        return ConnectionMapper.mapToConnection(ksmConnection)

    def getValidConnection(self, token):
        if token is None:
            raise NullClientConnectionException()

        ksmConnection = KsmConnectionModel(self.urlEngine).get(token)

        if ksmConnection is None:
            raise InvalidClientConnectionException()

        if ksmConnection.getActive() == False:
            raise ClientConnectionExpiredException()

        ksmConnection.setUpdatedAt(datetime.now())
        KsmConnectionModel(self.urlEngine).update(ksmConnection)
        return ConnectionMapper.mapToConnection(ksmConnection)



from datetime import datetime
import uuid

from cerberus.dtos.SecurityHash import SecurityHash
from cerberus.model.KsmSecurityModel import KsmSecurityModel
from cerberus.mappers.SecurityHashMapper import SecurityHashMapper
from cerberus.exceptions.exceptions import InvalidParamException, RequiredParamException, SecurityHashExpiredException, InvalidSecurityHashException
from cerberus.services.AbstractService import AbstractService

class SecurityHashService(AbstractService):

    def __init__(self, url):
        super(SecurityHashService, self).__init__(url)

    def createHash(self, type):

        if type is None:
            raise RequiredParamException("type")

        ksmSecurityHashType = KsmSecurityModel(self.urlEngine).getSecurityHashType(type)

        if ksmSecurityHashType is None:
            raise InvalidParamException("type")

        ksmSecurityHash = KsmSecurityModel(self.urlEngine).addSecurityHash(type, False, True)

        return SecurityHashMapper.mapToSecurityHash(ksmSecurityHash);

    def validateHash(self, token):

        if token is None:
            raise RequiredParamException("token")

        ksmSecurityHash = KsmSecurityModel(self.urlEngine).getSecurityHash(token)
        if ksmSecurityHash is None:
            raise InvalidSecurityHashException()

        if ksmSecurityHash.getActive() == False or ksmSecurityHash.getUsed() == True:
            raise SecurityHashExpiredException()

        ksmSecurityHash.setUsed(True)
        KsmSecurityModel(self.urlEngine).update(ksmSecurityHash)
        return SecurityHashMapper.mapToSecurityHash(ksmSecurityHash);



from abc import ABC, abstractmethod
from sqlalchemy import create_engine
from sqlalchemy.orm import Session

class AbstractModelResource(ABC):

	def __init__(self, url):
		self.engine = create_engine(url)
		self.session = Session(self.engine)
		self.session.expire_on_commit=False

	def update(self, obj):
		self.session.commit()

	def insert(self, obj):
		self.session.add(obj)
		self.session.commit()



from cerberus.model.AbstractModelResource import AbstractModelResource
from cerberus.entities.entities import KsmConnection
from datetime import datetime

class KsmConnectionModel(AbstractModelResource):
	"""docstring for KsmConnectionModel"""
	def __init__(self, url):
		super(KsmConnectionModel, self).__init__(url)
		self.url = url

	def add(self, token, client):
		ksmConnection = KsmConnection(token, True, client.getId(), datetime.now(), datetime.now())
		self.insert(ksmConnection)
		return ksmConnection

	def get(self, token):
		connection = self.session.query(KsmConnection).filter(KsmConnection.id == token).first()
		return connection


from cerberus.model.AbstractModelResource import AbstractModelResource
from cerberus.entities.entities import KsmUserAuthenticationType, KsmUser, KsmRole, KsmRoleUser


class KsmUserModel(AbstractModelResource):
	"""docstring for KsmUserModel"""
	def __init__(self, url):
		super(KsmUserModel, self).__init__(url)
		self.url = url

	def addUser(self, id, createdAt, updatedAt):
		ksmUser = KsmUser()
		ksmUser.setId(id)
		ksmUser.setCreatedAt(createdAt)
		ksmUser.setUpdatedAt(updatedAt)
		self.insert(ksmUser)
		return ksmUser

	def getUserAuthenticationType(self, userName, authenticationTypeId):
		kuat = self.session.query(KsmUserAuthenticationType).filter(KsmUserAuthenticationType.username == userName).filter(KsmUserAuthenticationType.authentication_type_id == authenticationTypeId).first()
		return kuat

	def getUserAuthenticationTypeByUsername(self, userName):
		kuat = self.session.query(KsmUserAuthenticationType).filter(KsmUserAuthenticationType.username == userName).first()
		return kuat

	def getUserAuthenticationTypeByUserId(self, userId, authenticationTypeId):
		kuat = self.session.query(KsmUserAuthenticationType).filter(KsmUserAuthenticationType.user_id == userId).filter(KsmUserAuthenticationType.authentication_type_id == authenticationTypeId).first()
		return kuat

	def addUserAuthenticationType(authenticationTypeId, userId, userName, token, createdAt, updatedAt):
		ksmUserAuthenticationType = KsmUserAuthenticationType()
		ksmUserAuthenticationType.setAuthenticationTypeId(authenticationTypeId)
		ksmUserAuthenticationType.setUserId(userId)
		ksmUserAuthenticationType.setUserName(userName)
		ksmUserAuthenticationType.setCreatedAt(createdAt)
		ksmUserAuthenticationType.setUpdatedAt(updatedAt)
		ksmUserAuthenticationType.setToken(token)
		self.insert(ksmUserAuthenticationType)
		return ksmUserAuthenticationType

	def getUserById(id):
		ksmUser = self.session.query(KsmUser).filter(KsmUser.id == id).first()
		return ksmUser

	def addUserRole(self, id, roleId, createdAt, updatedAt):
		ksmRoleUser = KsmRoleUser()
		ksmRoleUser.setUserId(id)
		ksmRoleUser.setRoleId(roleId)
		ksmRoleUser.setCreatedAt(createdAt)
		ksmRoleUser.setUpdatedAt(updatedAt)
		self.insert(ksmRoleUser)
		return ksmRoleUser

	def getRoleById(id):
		ksmUser = self.session.query(KsmRole).filter(KsmRole.id == id).first()
		return ksmRole

	def getRolesByUserId(userId):
		rolesUser = self.session.query(KsmRoleUser).filter(KsmRoleUser.user_id == userId).all()
		return rolesUser



from cerberus.model.AbstractModelResource import AbstractModelResource
from cerberus.entities.entities import KsmClient

class KsmClientModel(AbstractModelResource):
	"""docstring for KsmClientModel"""
	def __init__(self, url):
		super(KsmClientModel, self).__init__(url)
		self.url = url

	def getClientByUsername(self, username):
		client = self.session.query(KsmClient).filter(KsmClient.username == username).filter(KsmClient.active == True).first()
		return client



from sqlalchemy.sql.functions import user
from cerberus.model.AbstractModelResource import AbstractModelResource
from cerberus.entities.entities import KsmRoleUser
from datetime import datetime
import uuid

class KsmRoleUserModel(AbstractModelResource):

    def __init__(self, url):
        super(KsmRoleUserModel,self).__init__(url)
        self.url = url
    

    def getRoleByUserId(self,userId):
        role = self.session.query(KsmRoleUser).filter(KsmRoleUser.user_id==userId).first()

        if role is not None:
            return role
        else:
            print("Throw Exception")



from cerberus.model.AbstractModelResource import AbstractModelResource
from cerberus.entities.entities import KsmService, KsmClientService, KsmUserService, KsmRoleService

class KsmServiceModel(AbstractModelResource):
	"""docstring for KsmServiceModel"""
	def __init__(self, url):
		super(KsmServiceModel, self).__init__(url)
		self.url = url

	def getServiceByName(self, name):
		ksmService = self.session.query(KsmService).filter(KsmService.name == name).first()
		return ksmService

	def getClientServiceById(self, serviceId, clientId):
		ksmClientService = self.session.query(KsmClientService).filter(KsmClientService.client_id == clientId).filter(KsmClientService.service_id == serviceId).first()
		return ksmClientService

	def getUserServiceById(self, serviceId, userId):
		ksmUserService = self.session.query(KsmUserService).filter(KsmUserService.user_id == session.getUserId()).filter(KsmUserService.service_id == ksmClientService.getServiceId()).first()
		return ksmUserService

	def getRoleServiceById(self, serviceId, roleId):
		ksmRoleService = self.session.query(KsmRoleService).filter(KsmRoleService.role_id == roleId).filter(KsmRoleService.service_id == serviceId).first()
		return ksmRoleService
